==> home-made-io-fastmag-sync-m2-0.19.4-9537b4.zip: Process/Worker/ToMagento/Hydration/Product/Inventory.php <==
<?php
/*******************************************************************************
 * Created by HOMEMADE.IO SAS.
 * Fastmag Sync  -  Connect Fastmag with your Magento
 *
 * Copyright (C) HOMEMADE.IO SAS, Inc - All Rights Reserved
 *
 * @author    Simon Laubet-Xavier <simon.laubetxavier@home-made.io>
 * @copyright 2020-2021 HOMEMADE.IO SAS
 * @date      2021-08-06
 ******************************************************************************/

namespace Fastmag\Sync\Process\Worker\ToMagento\Hydration\Product;

use Exception;
use Fastmag\Sync\Exception\JobException;
use Fastmag\Sync\Model\Config;
use Fastmag\Sync\Process\Worker\ToMagento\Hydration\Product;
use Magento\Framework\Exception\CouldNotSaveException;

/**
 * Class Inventory
 *
 * Hydration class used for inserting or updating products inventories from Fastmag to Magento
 */
class Inventory extends Product
{
    /** @inheritDoc */
    protected $code = 'tomagento_hydration_product_inventory';

    /**
     * @inheritDoc
     */
    public function run()
    {
        foreach ($this->jobs->getItems() as $job) {
            $this->currentJob = $job;
            $this->defaultRule = null;

            $fastmagRef = $this->getCurrentFastmagRef();

            if ($this->getMagentoProductId($fastmagRef) !== false) {
                try {
                    $entity = $this->getDataFromFastmag();

                    $job->setHydratedData($entity->export())
                        ->setEntity($entity);
                } catch (Exception $exception) {
                    $job->traceException($exception);

                    $this->logger->error(
                        __(
                            '[Job #%1] Error when updating inventory level of product with Fastmag ref "%2": %3',
                            $job->getId(),
                            $fastmagRef,
                            $exception->getMessage()
                        )->render()
                    );
                }

                try {
                    $this->jobRepository->save($job);
                } catch (CouldNotSaveException $exception) {
                    $job->traceException($exception);
                }
            } else {
                $this->logger->notice(
                    __(
                        'Can not change inventory level of the product with Fastmag ref "%1", as it does not exist in Magento',
                        $fastmagRef
                    )->render()
                );
            }
        }
    }

    /**
     * @inheritDoc
     */
    protected function getDataFromFastmag()
    {
        $fastmagRef = $this->getCurrentFastmagRef();

        if ($this->isPack($fastmagRef)) {
            throw new JobException(__(
                'Product "%1" is a pack, which can not be imported with Fastmag_Sync module',
                $fastmagRef
            ));
        }

        $this->currentEntity = $this->productEntityFactory->create();
        $this->currentEntity->setRef($fastmagRef)
            ->setMagentoId($this->getMagentoProductId($fastmagRef));

        $this->getParentData();

        if ($this->productMustBeSynced()) {
            $storesList = $this->storeRepository->getList();

            foreach ($storesList as $store) {
                try {
                    $this->currentRule = $this->storesellerRuleRepository->getByStoreId($store->getId());
                } catch (Exception $exception) {
                    continue;
                }

                if ($this->defaultRule === null) {
                    $this->defaultRule = $this->currentRule;
                }

                $this->getChildrenData();
            }
        }

        return $this->currentEntity;
    }

    /**
     * @inheritDoc
     */
    protected function getParentData()
    {
        try {
            $sql = 'SELECT pf.VisibleWeb AS visible_web,
                    pf.Actif AS active,
                    COUNT(s.ID) AS stock_level,
                    GROUP_CONCAT(DISTINCT IF(Stock >= 1, s.CodeMag, NULL)) AS stocks_list,
                    f.TauxTVA AS vat_rate
                FROM produitsfiches AS pf LEFT JOIN stock AS s on pf.BarCode = s.BarCode
                    LEFT JOIN familles AS f ON pf.Famille = f.Famille
                WHERE pf.BarCode = '
                    . $this->getFastmagSqlConnection()->escape($this->currentEntity->getRef(), true) . '
                    AND s.AR = 1
                GROUP BY pf.BarCode';

            $rows = $this->getFastmagSqlConnection()->get($sql);

            if (count($rows) === 0) {
                $sql = 'SELECT pf.VisibleWeb AS visible_web,
                    pf.Actif AS active,
                    0 AS stock_level,
                    \'\' AS stocks_list,
                    f.TauxTVA AS vat_rate
                FROM produitsfiches AS pf LEFT JOIN stock AS s on pf.BarCode = s.BarCode
                    LEFT JOIN familles AS f ON pf.Famille = f.Famille
                WHERE pf.BarCode = '
                    . $this->getFastmagSqlConnection()->escape($this->currentEntity->getRef(), true) . '
                GROUP BY pf.BarCode';

                $rows = $this->getFastmagSqlConnection()->get($sql);
            }
        } catch (Exception $exception) {
            throw new JobException(__($exception->getMessage()));
        }

        $row = reset($rows);

        $this->currentEntity->addData($row);
    }

    /**
     * @inheritDoc
     */
    protected function getChildrenDataSql()
    {
        $onlyStoresellerCombination = '';
        if ($this->config->isSetFlag(Config::XML_PATH_PRODUCT_IMPORT_ONLY_STORESELLER)) {
            $onlyStoresellerCombination = 'HAVING active_stock = 1 ';
        }

        $stockCondition = '';
        $referenceStocks = $this->getReferenceStocks();
        if ($referenceStocks !== null && count($referenceStocks) !== 0) {
            $stockCondition = ' AND CodeMag IN ('
                . $this->getFastmagSqlConnection()->escape($referenceStocks) . ')';
        }

        return 'SELECT
                (
                    SELECT CONCAT(
                        \'{"\', \'fastmag_id\', \'": \', Produit,
                        \', "\', \'ean13\', \'": "\', RIGHT(GenCod, 13),
                        \'", "\', \'supplier_ref\', \'": "\', RefFournisseur,
                        \'", "\', \'weight\', \'": \', IF(PoidsTC > 0, PoidsTC, 0.1), \'}\'
                    )
                    FROM produits AS p
                    WHERE p.BarCode = stock.BarCode AND p.Taille = stock.Taille AND p.Couleur = stock.Couleur
                    LIMIT 1
                ) AS combination_data,
                (
                    SELECT 1
                    FROM stock AS s2
                    WHERE s2.BarCode = stock.BarCode AND s2.Taille = stock.Taille AND s2.Couleur = stock.Couleur
                        ' . $stockCondition . '
                    LIMIT 1
                ) AS active_stock,
                stock.Taille AS size,
                stock.Couleur AS color,
                SUM(Stock) AS stock_level,
                stock.AR AS in_stock
            FROM produitsfiches AS pf INNER JOIN stock ON pf.BarCode = stock.BarCode
            WHERE stock.AR = 1
                AND stock.BarCode = 
                ' . $this->getFastmagSqlConnection()->escape($this->currentEntity->getRef(), true) . '
                ' . $stockCondition . '
            GROUP BY stock.Barcode, stock.Taille, stock.Couleur
            ' . $onlyStoresellerCombination . '
            ORDER BY stock.Couleur, stock.Taille';
    }
}

==> home-made-io-fastmag-sync-m2-0.24.0-762a73.zip: Process/Worker/ToMagento/Hydration/Product/Inventory.php <==
<?php
/*******************************************************************************
 * Created by HOMEMADE.IO SAS.
 * Fastmag Sync  -  Connect Fastmag with your Magento
 *
 * Copyright (C) HOMEMADE.IO SAS, Inc - All Rights Reserved
 *
 * @author    Simon Laubet-Xavier <simon.laubetxavier@home-made.io>
 * @copyright 2020-2021 HOMEMADE.IO SAS
 * @date      2021-09-29
 ******************************************************************************/

namespace Fastmag\Sync\Process\Worker\ToMagento\Hydration\Product;

use Exception;
use Fastmag\Sync\Exception\JobException;
use Fastmag\Sync\Exception\ProductNotSyncableException;
use Fastmag\Sync\Model\Config;
use Fastmag\Sync\Process\Worker\ToMagento\Hydration\Product;

/**
 * Class Inventory
 *
 * Hydration class used for inserting or updating products inventories from Fastmag to Magento
 */
class Inventory extends Product
{
    /** @inheritDoc */
    protected $code = 'tomagento_hydration_product_inventory';

    /** @inheritDoc */
    protected $syncConfigFields = [
        Config::XML_PATH_PRODUCT_IMPORT_ONLY_DEFINED_STOCKS,
        Config::XML_PATH_INVENTORY_FASTMAG_STOCK_REFERENCE_STOCK,
        Config::XML_PATH_INVENTORY_FASTMAG_STOCK_ALTERNATIVE_STOCKS
    ];

    /**
     * @inheritDoc
     */
    public function run()
    {
        $this->getSyncConfig();

        foreach ($this->getJobs()->getItems() as $job) {
            $this->currentJob = $job;

            $fastmagRef = $this->currentJob->getContentId();

            if ($this->getMagentoProductId($fastmagRef) !== false) {
                try {
                    $entity = $this->getDataFromFastmag();

                    $this->hydrateJob($job, $entity);
                } catch (JobException $exception) {
                    $this->invalidateJob($job, $exception);
                }

                $this->saveJob($job);
            } else {
                $exception = new ProductNotSyncableException(__(
                    'Can not change inventory level of the product with Fastmag ref "%1", as it does not exist in Magento',
                    $fastmagRef
                ));

                $this->skipJob($job, $exception);
            }
        }
    }

    /**
     * @inheritDoc
     */
    protected function getParentData()
    {
        try {
            $sql = 'SELECT pf.VisibleWeb AS visible_web,
                    pf.Actif AS active,
                    COUNT(s.ID) AS stock_level,
                    GROUP_CONCAT(DISTINCT IF(Stock >= 1, s.CodeMag, NULL)) AS stocks_list
                FROM produitsfiches AS pf LEFT JOIN stock AS s on pf.BarCode = s.BarCode
                    LEFT JOIN familles AS f ON pf.Famille = f.Famille
                WHERE pf.BarCode = '
                    . $this->getFastmagSqlConnection()->escape($this->currentEntity->getRef(), true) . '
                    AND s.AR = 1
                GROUP BY pf.BarCode';

            $rows = $this->getFastmagSqlConnection()->get($sql);

            if (count($rows) === 0) {
                $sql = 'SELECT pf.VisibleWeb AS visible_web,
                    pf.Actif AS active,
                    0 AS stock_level,
                    \'\' AS stocks_list
                FROM produitsfiches AS pf LEFT JOIN stock AS s on pf.BarCode = s.BarCode
                    LEFT JOIN familles AS f ON pf.Famille = f.Famille
                WHERE pf.BarCode = '
                    . $this->getFastmagSqlConnection()->escape($this->currentEntity->getRef(), true) . '
                GROUP BY pf.BarCode';

                $rows = $this->getFastmagSqlConnection()->get($sql);
            }
        } catch (Exception $exception) {
            throw new JobException(__($exception->getMessage()));
        }

        $row = reset($rows);

        $this->currentEntity->addData($row);
    }

    /**
     * @inheritDoc
     */
    protected function getChildrenDataSql($storeId = null)
    {
        $stockCondition = $this->getStocksCodesCondition($storeId);

        return 'SELECT
                (
                    SELECT CONCAT(
                        \'{"\', \'fastmag_id\', \'": \', Produit,
                        \', "\', \'ean13\', \'": "\', RIGHT(GenCod, 13),
                        \'", "\', \'supplier_ref\', \'": "\', RefFournisseur,
                        \'", "\', \'weight\', \'": \', IF(PoidsTC > 0, PoidsTC, 0.1), \'}\'
                    )
                    FROM produits AS p
                    WHERE p.BarCode = stock.BarCode AND p.Taille = stock.Taille AND p.Couleur = stock.Couleur
                    LIMIT 1
                ) AS combination_data,
                (
                    SELECT 1
                    FROM stock AS s2
                    WHERE s2.BarCode = stock.BarCode AND s2.Taille = stock.Taille AND s2.Couleur = stock.Couleur
                        ' . $stockCondition . '
                    LIMIT 1
                ) AS active_stock,
                stock.Couleur AS color,
                stock.Taille AS size,
                (
                    SELECT TRIM(Descriptif)
                    FROM produitscouleurs AS pc
                    WHERE pc.Barcode = stock.BarCode AND pc.Couleur = stock.Couleur
                ) AS color_description,
                SUM(Stock) AS stock_level,
                stock.AR AS in_stock
            FROM produitsfiches AS pf INNER JOIN stock ON pf.BarCode = stock.BarCode
            WHERE stock.AR = 1
                AND stock.BarCode = 
                ' . $this->getFastmagSqlConnection()->escape($this->currentEntity->getRef(), true) . '
                ' . $stockCondition . '
            GROUP BY stock.Barcode, stock.Taille, stock.Couleur
            ' . $this->getOnlyDefinedStocksCondition($storeId) . '
            ORDER BY stock.Couleur, stock.Taille';
    }
}
